@@ -35,7 +35,6 @@
 #include "gtksourcebuffer.h"
 #include "gtksourceundomanager.h"
 #include "gtksourceview-marshal.h"
-#include "gtksourceiter.h"
 #include "gtksourcestyleschememanager.h"
 #include "gtksourcestyle-private.h"
 #include "gtksourceundomanagerdefault.h"
@@ -362,7 +361,8 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	 * @state: state of bracket matching
 	 *
 	 * Sets @iter to a valid iterator pointing to the matching bracket
-	 * if @state is #GTK_SOURCE_BRACKET_MATCH_FOUND.
+	 * if @state is #GTK_SOURCE_BRACKET_MATCH_FOUND. Otherwise @iter is
+	 * meaningless.
 	 *
 	 * Since: 2.12
 	 */
@@ -788,6 +788,7 @@ gtk_source_buffer_move_cursor (GtkTextBuffer     *buffer,
 	GtkSourceBuffer *source_buffer;
 	GtkTextIter start, end;
 	gunichar cursor_char;
+	GtkSourceBracketMatchType previous_state;
 
 	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
 	g_return_if_fail (iter != NULL);
@@ -821,16 +822,23 @@ gtk_source_buffer_move_cursor (GtkTextBuffer     *buffer,
 		return;
 
 	start = *iter;
+	previous_state = source_buffer->priv->bracket_match;
 	if (!gtk_source_buffer_find_bracket_match_with_limit (source_buffer,
 	                                                      &start,
 	                                                      &source_buffer->priv->bracket_match,
 	                                                      MAX_CHARS_BEFORE_FINDING_A_MATCH))
 	{
-		g_signal_emit (source_buffer,
-			       buffer_signals[BRACKET_MATCHED],
-			       0,
-			       NULL,
-			       &source_buffer->priv->bracket_match);
+		/* don't emit the signal at all if chars at previous and current
+		   positions are nonbrackets. */
+		if (previous_state != GTK_SOURCE_BRACKET_MATCH_NONE ||
+		    source_buffer->priv->bracket_match != GTK_SOURCE_BRACKET_MATCH_NONE)
+		{
+			g_signal_emit (source_buffer,
+				       buffer_signals[BRACKET_MATCHED],
+				       0,
+				       &end,
+				       source_buffer->priv->bracket_match);
+		}
 	}
 	else
 	{
@@ -838,7 +846,7 @@ gtk_source_buffer_move_cursor (GtkTextBuffer     *buffer,
 			       buffer_signals[BRACKET_MATCHED],
 			       0,
 			       &start,
-			       &source_buffer->priv->bracket_match);
+			       source_buffer->priv->bracket_match);
 
 		/* allow_bracket_match will allow the bracket match tag to be
 		   applied to the buffer. See apply_tag_real for more
@@ -2067,7 +2075,8 @@ gtk_source_buffer_backward_iter_to_source_mark (GtkSourceBuffer *buffer,
  * Returns the list of marks of the given category at @iter. If @category
  * is %NULL it returns all marks at @iter.
  *
- * Returns: (element-type GtkSource.Mark): a newly allocated #GSList.
+ * Returns: (element-type GtkSource.Mark) (transfer container):
+ * a newly allocated #GSList.
  *
  * Since: 2.2
  **/
@@ -2112,7 +2121,8 @@ gtk_source_buffer_get_source_marks_at_iter (GtkSourceBuffer *buffer,
  * Returns the list of marks of the given category at @line.
  * If @category is NULL, all marks at @line are returned.
  *
- * Returns: (element-type GtkSource.Mark): a newly allocated #GSList.
+ * Returns: (element-type GtkSource.Mark) (transfer container):
+ * a newly allocated #GSList.
  *
  * Since: 2.2
  **/
@@ -2262,8 +2272,9 @@ gtk_source_buffer_iter_has_context_class (GtkSourceBuffer   *buffer,
  *
  * Get all defined context classes at @iter.
  *
- * Returns: (array zero-terminated=1): a new %NULL terminated array of context
- * class names. Use #g_strfreev to free the array if it is no longer needed.
+ * Returns: (array zero-terminated=1) (transfer full): a new %NULL
+ * terminated array of context class names.
+ * Use g_strfreev() to free the array if it is no longer needed.
  *
  * Since: 2.10
  **/
