@@ -1,15 +1,15 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*- /
  * gtksourcegutter.c
- * This file is part of gtksourceview
+ * This file is part of GtkSourceView
  *
  * Copyright (C) 2009 - Jesse van den Kieboom
  *
- * gtksourceview is free software; you can redistribute it and/or
+ * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
  *
- * gtksourceview is distributed in the hope that it will be useful,
+ * GtkSourceView is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
@@ -24,6 +24,37 @@
 #include "gtksourceview-i18n.h"
 #include "gtksourceview-marshal.h"
 
+/**
+ * SECTION:gutter
+ * @Short_description: Gutter object for #GtkSourceView
+ * @Title: GtkSourceGutter
+ * @See_also:#GtkSourceView
+ *
+ * The #GtkSourceGutter object represents the left and right gutters of the text
+ * view. It is used by #GtkSourceView to draw the line numbers and category
+ * marks that might be present on a line. By packing additional #GtkCellRenderer
+ * objects in the gutter, you can extend the gutter with your own custom 
+ * drawings.
+ *
+ * The gutter works very much the same way as cells rendered in a #GtkTreeView.
+ * The concept is similar, with the exception that the gutter does not have an
+ * underlying #GtkTreeModel. Instead, you should use
+ * #gtk_source_gutter_set_cell_data_func to set a callback to fill in any of the
+ * cell renderers properties, given the line for which the cell is to be
+ * rendered. Renderers are inserted into the gutter at a certain position. The
+ * builtin line number renderer is at position 
+
+ * #GTK_SOURCE_VIEW_GUTTER_POSITION_LINES (-30) and the marks renderer is at
+ * #GTK_SOURCE_VIEW_GUTTER_POSITION_MARKS (-20). You can use these values to
+ * position custom renderers accordingly. The width of a cell renderer can be
+ * specified as either fixed (using
+
+ * #gtk_cell_renderer_set_fixed_size) or dynamic, in which case you
+ * <emphasis>must</emphasis> set #gtk_source_gutter_set_cell_size_func. This
+ * callback is used to set the properties of the renderer such that
+ * #gtk_cell_size_request_get_size yields the maximum width of the cell.
+ */
+
 #define GTK_SOURCE_GUTTER_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_TYPE_SOURCE_GUTTER, GtkSourceGutterPrivate))
 
 /* Properties */
@@ -60,7 +91,7 @@ typedef struct
 
 enum
 {
-	EXPOSE_EVENT,
+	DRAW,
 	MOTION_NOTIFY_EVENT,
 	BUTTON_PRESS_EVENT,
 	ENTER_NOTIFY_EVENT,
@@ -80,9 +111,9 @@ struct _GtkSourceGutterPrivate
 
 G_DEFINE_TYPE (GtkSourceGutter, gtk_source_gutter, G_TYPE_OBJECT)
 
-static gboolean on_view_expose_event (GtkSourceView   *view,
-                                      GdkEventExpose  *event,
-                                      GtkSourceGutter *gutter);
+static gboolean on_view_draw (GtkSourceView   *view,
+                              cairo_t         *cr,
+                              GtkSourceGutter *gutter);
 
 static gboolean on_view_motion_notify_event (GtkSourceView   *view,
                                              GdkEventMotion  *event,
@@ -211,10 +242,10 @@ set_view (GtkSourceGutter *gutter,
 	                   (GWeakNotify)view_notify,
 	                   gutter);
 
-	gutter->priv->signals[EXPOSE_EVENT] =
+	gutter->priv->signals[DRAW] =
 		g_signal_connect (view,
-	                  "expose-event",
-	                  G_CALLBACK (on_view_expose_event),
+	                  "draw",
+	                  G_CALLBACK (on_view_draw),
 	                  gutter);
 
 	gutter->priv->signals[MOTION_NOTIFY_EVENT] =
@@ -272,7 +303,7 @@ revalidate_size (GtkSourceGutter *gutter)
 	if (!window && gutter->priv->renderers)
 	{
 		/* Make window visible by setting its size to minimum size,
-		   actual size will be calculated in expose */
+		   actual size will be calculated in draw */
 		gtk_text_view_set_border_window_size (GTK_TEXT_VIEW (gutter->priv->view),
 		                                      gutter->priv->window_type,
 		                                      1);
@@ -287,7 +318,7 @@ revalidate_size (GtkSourceGutter *gutter)
 	}
 	else if (window)
 	{
-		/* Redraw the window. Actual size will be calculated in expose */
+		/* Redraw the window. Actual size will be calculated in draw */
 		do_redraw (gutter);
 	}
 }
@@ -377,7 +408,7 @@ gtk_source_gutter_class_init (GtkSourceGutterClass *klass)
 			      3,
 			      GTK_TYPE_CELL_RENDERER,
 			      GTK_TYPE_TEXT_ITER,
-			      G_TYPE_POINTER);
+			      GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
 
 	/**
 	 * GtkSourceGutter::query-tooltip:
@@ -459,12 +490,13 @@ gtk_source_gutter_new (GtkSourceView     *view,
 
 /**
  * gtk_source_gutter_get_window:
- * @gutter: a #GtkSourceGutter
+ * @gutter: a #GtkSourceGutter.
  *
  * Get the #GdkWindow of the gutter. The window will only be available when the
  * gutter has at least one, non-zero width, cell renderer packed.
  *
- * Returns: the #GdkWindow of the gutter, or %NULL if the gutter has no window.
+ * Returns: (transfer none): the #GdkWindow of the gutter, or %NULL
+ * if the gutter has no window.
  *
  * Since: 2.8
  */
@@ -480,9 +512,9 @@ gtk_source_gutter_get_window (GtkSourceGutter *gutter)
 
 /**
  * gtk_source_gutter_insert:
- * @gutter: a #GtkSourceGutter
- * @renderer: a #GtkCellRenderer
- * @position: the renderers position
+ * @gutter: a #GtkSourceGutter.
+ * @renderer: a #GtkCellRenderer.
+ * @position: the renderers position.
  *
  * Inserts @renderer into @gutter at @position.
  *
@@ -528,9 +560,9 @@ renderer_find (GtkSourceGutter  *gutter,
 
 /**
  * gtk_source_gutter_reorder:
- * @gutter: a #GtkSourceGutter
- * @renderer: a #GtkCellRenderer
- * @position: the new renderer position
+ * @gutter: a #GtkSourceGutter.
+ * @renderer: a #GtkCellRenderer.
+ * @position: the new renderer position.
  *
  * Reorders @renderer in @gutter to new @position.
  *
@@ -557,8 +589,8 @@ gtk_source_gutter_reorder (GtkSourceGutter *gutter,
 
 /**
  * gtk_source_gutter_remove:
- * @gutter: a #GtkSourceGutter
- * @renderer: a #GtkCellRenderer
+ * @gutter: a #GtkSourceGutter.
+ * @renderer: a #GtkCellRenderer.
  *
  * Removes @renderer from @gutter.
  *
@@ -585,11 +617,11 @@ gtk_source_gutter_remove (GtkSourceGutter *gutter,
 
 /**
  * gtk_source_gutter_set_cell_data_func:
- * @gutter: a #GtkSourceGutter
- * @renderer: a #GtkCellRenderer
- * @func: the #GtkSourceGutterDataFunc to use
- * @func_data: the user data for @func
- * @destroy: the destroy notification for @func_data
+ * @gutter: a #GtkSourceGutter.
+ * @renderer: a #GtkCellRenderer.
+ * @func: (allow-none): the #GtkSourceGutterDataFunc to use.
+ * @func_data: (allow-none): the user data for @func.
+ * @destroy: (allow-none): the destroy notification for @func_data.
  *
  * Sets the #GtkSourceGutterDataFunc to use for @renderer. This function is
  * used to setup the cell renderer properties for rendering the current cell.
@@ -627,11 +659,11 @@ gtk_source_gutter_set_cell_data_func (GtkSourceGutter         *gutter,
 
 /**
  * gtk_source_gutter_set_cell_size_func:
- * @gutter: a #GtkSourceGutter
- * @renderer: a #GtkCellRenderer
- * @func: the #GtkSourceGutterSizeFunc to use
- * @func_data: the user data for @func
- * @destroy: the destroy notification for @func_data
+ * @gutter: a #GtkSourceGutter.
+ * @renderer: a #GtkCellRenderer.
+ * @func: (allow-none): the #GtkSourceGutterSizeFunc to use.
+ * @func_data: (allow-none): the user data for @func.
+ * @destroy: (allow-none): the destroy notification for @func_data.
  *
  * Sets the #GtkSourceGutterSizeFunc to use for @renderer. This function is
  * used to setup the cell renderer properties for measuring the maximum size
@@ -670,7 +702,7 @@ gtk_source_gutter_set_cell_size_func (GtkSourceGutter         *gutter,
 
 /**
  * gtk_source_gutter_queue_draw:
- * @gutter: a #GtkSourceGutter
+ * @gutter: a #GtkSourceGutter.
  *
  * Invalidates the drawable area of the gutter. You can use this to force a
  * redraw of the gutter if something has changed and needs to be redrawn.
@@ -695,13 +727,13 @@ calculate_size (GtkSourceGutter  *gutter,
 
 	if (width == -1 && renderer->size_func)
 	{
-		gint height;
+                GtkRequisition req;
 		renderer->size_func (gutter, renderer->renderer, renderer->size_func_data);
 
-		gtk_cell_renderer_get_size (renderer->renderer,
-		                            GTK_WIDGET (gutter->priv->view),
-		                            NULL, NULL, NULL,
-		                            &width, &height);
+		gtk_cell_renderer_get_preferred_size (renderer->renderer,
+                                                      GTK_WIDGET (gutter->priv->view),
+                                                      &req, NULL);
+                width = req.width;
 	}
 
 	return width == -1 ? 1 : width;
@@ -827,29 +859,32 @@ get_lines (GtkTextView  *text_view,
 }
 
 static gboolean
-on_view_expose_event (GtkSourceView   *view,
-                      GdkEventExpose  *event,
-                      GtkSourceGutter *gutter)
+on_view_draw (GtkSourceView   *view,
+              cairo_t         *cr,
+              GtkSourceGutter *gutter)
 {
 	GdkWindow *window;
 	GtkTextView *text_view;
 	GArray *sizes;
+        GdkRectangle clip;
 	gint size;
 
 	window = gtk_source_gutter_get_window (gutter);
 
-	if (window == NULL || event->window != window)
+	if (window == NULL || !gtk_cairo_should_draw_window (cr, window))
 	{
 		return FALSE;
 	}
 
+        gtk_cairo_transform_to_window (cr, GTK_WIDGET (view), window);
+
 	text_view = GTK_TEXT_VIEW (view);
 	sizes = g_array_new (FALSE, FALSE, sizeof (gint));
 
 	/* This is fairly ugly, but we could not find a better way to
 	 * do it: renderers could have changed size and they do not have
-	 * a way to signal it. So on expose we revalidate the size and
-	 * requeue another expose.
+	 * a way to signal it. So on draw we revalidate the size and
+	 * requeue another draw.
 	 * To see if the size has changed we test the size of only the
 	 * gutter itself since the full border window size is not under
 	 * our control (see e.g bug #589382).
@@ -859,7 +894,6 @@ on_view_expose_event (GtkSourceView   *view,
 	size = calculate_sizes (gutter, sizes);
 	if (gutter->priv->size != size)
 	{
-
 		gint border_size;
 
 		border_size = gtk_text_view_get_border_window_size (text_view, gutter->priv->window_type);
@@ -871,11 +905,12 @@ on_view_expose_event (GtkSourceView   *view,
 
 		gutter->priv->size = size;
 
-		/* Will trigger a new expose */
+		/* Will trigger a new draw */
 		gtk_text_view_set_border_window_size (text_view, gutter->priv->window_type, border_size);
 	}
-	else
+	else if (gdk_cairo_get_clip_rectangle (cr, &clip))
 	{
+		GtkTextBuffer *buffer = gtk_text_view_get_buffer (text_view);
 		gint x, y;
 		gint y1, y2;
 		GArray *numbers;
@@ -889,8 +924,8 @@ on_view_expose_event (GtkSourceView   *view,
 
 		gdk_window_get_pointer (window, &x, &y, NULL);
 
-		y1 = event->area.y;
-		y2 = y1 + event->area.height;
+		y1 = clip.y;
+		y2 = y1 + clip.height;
 
 		/* get the extents of the line printing */
 		gtk_text_view_window_to_buffer_coords (text_view,
@@ -914,9 +949,9 @@ on_view_expose_event (GtkSourceView   *view,
 		/* get the line numbers and y coordinates. */
 		get_lines (text_view, y1, y2, pixels, heights, numbers, &count);
 
-		gtk_text_buffer_get_iter_at_mark (text_view->buffer,
+		gtk_text_buffer_get_iter_at_mark (buffer,
 						  &cur,
-						  gtk_text_buffer_get_insert (text_view->buffer));
+						  gtk_text_buffer_get_insert (buffer));
 
 		cur_line = gtk_text_iter_get_line (&cur);
 
@@ -973,11 +1008,10 @@ on_view_expose_event (GtkSourceView   *view,
 
 				/* Call render with correct area */
 				gtk_cell_renderer_render (renderer->renderer,
-					                  window,
+					                  cr,
 					                  GTK_WIDGET (view),
 					                  &cell_area,
 					                  &cell_area,
-					                  &cell_area,
 					                  state);
 
 				cell_area.x += cell_area.width;
@@ -1158,7 +1192,18 @@ on_view_button_press_event (GtkSourceView    *view,
 		g_free (path);
 		do_redraw (gutter);
 
-		return ret;
+		/* WARNING, DANGER, DANGER:
+		 * FIXME: this is really just a hack because we lack a return
+		 * value in the CELL_ACTIVATED signal, so handlers of cell
+		 * activating have no way to consume the button event. A left
+		 * click in the gutter will by default remove the selection
+		 * in the textview, which is bad because we implement some
+		 * selection stuff in GtkSourceView when clicking on the line
+		 * renderer. Anyway, for 3.0 we should change the
+		 * cell-activated signal to allow a return value and remove
+		 * this hack
+		 */
+		return ret || event->button == 1;
 	}
 
 	return FALSE;
