@@ -2,7 +2,7 @@
 
 
 #ifndef __GTKSOURCEVIEW_TYPEBUILTINS_H__
-#define __GTKSOURCEVIEW_TYPEBUILTINS_H__ 1
+#define __GTKSOURCEVIEW_TYPEBUILTINS_H__
 
 #include <gtksourceview/gtksourcebuffer.h>
 G_BEGIN_DECLS
@@ -10,10 +10,16 @@ G_BEGIN_DECLS
 GType gtk_source_bracket_match_type_get_type (void);
 G_END_DECLS
 
-#include <gtksourceview/gtksourceiter.h>
+#include <gtksourceview/gtksourcecompletioncontext.h>
 G_BEGIN_DECLS
-#define GTK_TYPE_SOURCE_SEARCH_FLAGS gtk_source_search_flags_get_type()
-GType gtk_source_search_flags_get_type (void);
+#define GTK_TYPE_SOURCE_COMPLETION_ACTIVATION gtk_source_completion_activation_get_type()
+GType gtk_source_completion_activation_get_type (void);
+G_END_DECLS
+
+#include <gtksourceview/gtksourcecompletion.h>
+G_BEGIN_DECLS
+#define GTK_TYPE_SOURCE_COMPLETION_ERROR gtk_source_completion_error_get_type()
+GType gtk_source_completion_error_get_type (void);
 G_END_DECLS
 
 #include <gtksourceview/gtksourceview.h>
@@ -32,18 +38,6 @@ G_BEGIN_DECLS
 GType gtk_source_draw_spaces_flags_get_type (void);
 G_END_DECLS
 
-#include <gtksourceview/gtksourcecompletion.h>
-G_BEGIN_DECLS
-#define GTK_TYPE_SOURCE_COMPLETION_ERROR gtk_source_completion_error_get_type()
-GType gtk_source_completion_error_get_type (void);
-G_END_DECLS
-
-#include <gtksourceview/gtksourcecompletioncontext.h>
-G_BEGIN_DECLS
-#define GTK_TYPE_SOURCE_COMPLETION_ACTIVATION gtk_source_completion_activation_get_type()
-GType gtk_source_completion_activation_get_type (void);
-G_END_DECLS
-
 #endif /* __GTKSOURCEVIEW_TYPEBUILTINS_H__ */
 
 
